# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, Annotated, TypedDict

from .volume import Volume
from ...._utils import PropertyInfo
from .url_param import URLParam
from .use_case_categories import UseCaseCategories
from .tf_phone_number_param import TfPhoneNumberParam
from .toll_free_verification_entity_type import TollFreeVerificationEntityType

__all__ = ["RequestCreateParams"]


class RequestCreateParams(TypedDict, total=False):
    additional_information: Required[Annotated[str, PropertyInfo(alias="additionalInformation")]]
    """Any additional information"""

    business_addr1: Required[Annotated[str, PropertyInfo(alias="businessAddr1")]]
    """Line 1 of the business address"""

    business_city: Required[Annotated[str, PropertyInfo(alias="businessCity")]]
    """The city of the business address; the first letter should be capitalized"""

    business_contact_email: Required[Annotated[str, PropertyInfo(alias="businessContactEmail")]]
    """The email address of the business contact"""

    business_contact_first_name: Required[Annotated[str, PropertyInfo(alias="businessContactFirstName")]]
    """
    First name of the business contact; there are no specific requirements on
    formatting
    """

    business_contact_last_name: Required[Annotated[str, PropertyInfo(alias="businessContactLastName")]]
    """
    Last name of the business contact; there are no specific requirements on
    formatting
    """

    business_contact_phone: Required[Annotated[str, PropertyInfo(alias="businessContactPhone")]]
    """The phone number of the business contact in E.164 format"""

    business_name: Required[Annotated[str, PropertyInfo(alias="businessName")]]
    """Name of the business; there are no specific formatting requirements"""

    business_state: Required[Annotated[str, PropertyInfo(alias="businessState")]]
    """
    The full name of the state (not the 2 letter code) of the business address; the
    first letter should be capitalized
    """

    business_zip: Required[Annotated[str, PropertyInfo(alias="businessZip")]]
    """The ZIP code of the business address"""

    corporate_website: Required[Annotated[str, PropertyInfo(alias="corporateWebsite")]]
    """A URL, including the scheme, pointing to the corporate website"""

    isv_reseller: Required[Annotated[str, PropertyInfo(alias="isvReseller")]]
    """ISV name"""

    message_volume: Required[Annotated[Volume, PropertyInfo(alias="messageVolume")]]
    """Message Volume Enums"""

    opt_in_workflow: Required[Annotated[str, PropertyInfo(alias="optInWorkflow")]]
    """
    Human-readable description of how end users will opt into receiving messages
    from the given phone numbers
    """

    opt_in_workflow_image_urls: Required[Annotated[Iterable[URLParam], PropertyInfo(alias="optInWorkflowImageURLs")]]
    """Images showing the opt-in workflow"""

    phone_numbers: Required[Annotated[Iterable[TfPhoneNumberParam], PropertyInfo(alias="phoneNumbers")]]
    """The phone numbers to request the verification of"""

    production_message_content: Required[Annotated[str, PropertyInfo(alias="productionMessageContent")]]
    """An example of a message that will be sent from the given phone numbers"""

    use_case: Required[Annotated[UseCaseCategories, PropertyInfo(alias="useCase")]]
    """Tollfree usecase categories"""

    use_case_summary: Required[Annotated[str, PropertyInfo(alias="useCaseSummary")]]
    """Human-readable summary of the desired use-case"""

    age_gated_content: Annotated[bool, PropertyInfo(alias="ageGatedContent")]
    """Indicates if messaging content requires age gating (e.g., 18+).

    Defaults to false if not provided.
    """

    business_addr2: Annotated[str, PropertyInfo(alias="businessAddr2")]
    """Line 2 of the business address"""

    business_registration_country: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationCountry")]
    """ISO 3166-1 alpha-2 country code of the issuing business authority.

    Must be exactly 2 letters. Automatically converted to uppercase. Required from
    January 2026.
    """

    business_registration_number: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationNumber")]
    """
    Official business registration number (e.g., Employer Identification Number
    (EIN) in the U.S.). Required from January 2026.
    """

    business_registration_type: Annotated[Optional[str], PropertyInfo(alias="businessRegistrationType")]
    """Type of business registration being provided. Required from January 2026."""

    campaign_verify_authorization_token: Annotated[
        Optional[str], PropertyInfo(alias="campaignVerifyAuthorizationToken")
    ]
    """
    Campaign Verify Authorization Token required for Political use case submissions
    starting February 17, 2026. This token is validated by Zipwhip and must be
    provided for all Political use case verifications after the deadline.
    """

    doing_business_as: Annotated[Optional[str], PropertyInfo(alias="doingBusinessAs")]
    """Doing Business As (DBA) name if different from legal name"""

    entity_type: Annotated[Optional[TollFreeVerificationEntityType], PropertyInfo(alias="entityType")]
    """Business entity classification"""

    help_message_response: Annotated[Optional[str], PropertyInfo(alias="helpMessageResponse")]
    """The message returned when users text 'HELP'"""

    opt_in_confirmation_response: Annotated[Optional[str], PropertyInfo(alias="optInConfirmationResponse")]
    """Message sent to users confirming their opt-in to receive messages"""

    opt_in_keywords: Annotated[Optional[str], PropertyInfo(alias="optInKeywords")]
    """Keywords used to collect and process consumer opt-ins"""

    privacy_policy_url: Annotated[Optional[str], PropertyInfo(alias="privacyPolicyURL")]
    """URL pointing to the business's privacy policy.

    Plain string, no URL format validation.
    """

    terms_and_condition_url: Annotated[Optional[str], PropertyInfo(alias="termsAndConditionURL")]
    """URL pointing to the business's terms and conditions.

    Plain string, no URL format validation.
    """

    webhook_url: Annotated[str, PropertyInfo(alias="webhookUrl")]
    """URL that should receive webhooks relating to this verification request"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel
from .mdr_detail_report_response import MdrDetailReportResponse

__all__ = ["MessagingDeleteResponse"]


class MessagingDeleteResponse(BaseModel):
    data: Optional[MdrDetailReportResponse] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CallControlApplicationOutboundParam"]


class CallControlApplicationOutboundParam(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the total number of outbound calls to phone numbers
    associated with this connection.
    """

    outbound_voice_profile_id: str
    """Identifies the associated outbound voice profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "RcsSuggestion",
    "Action",
    "ActionCreateCalendarEventAction",
    "ActionDialAction",
    "ActionOpenURLAction",
    "ActionViewLocationAction",
    "ActionViewLocationActionLatLong",
    "Reply",
]


class ActionCreateCalendarEventAction(BaseModel):
    """
    Opens the user's default calendar app and starts the new calendar event flow with the agent-specified event data pre-filled.
    """

    description: Optional[str] = None
    """Event description. Maximum 500 characters."""

    end_time: Optional[datetime] = None

    start_time: Optional[datetime] = None

    title: Optional[str] = None
    """Event title. Maximum 100 characters."""


class ActionDialAction(BaseModel):
    """
    Opens the user's default dialer app with the agent-specified phone number filled in.
    """

    phone_number: str
    """Phone number in +E.164 format"""


class ActionOpenURLAction(BaseModel):
    """Opens the user's default web browser app to the specified URL."""

    application: Literal["OPEN_URL_APPLICATION_UNSPECIFIED", "BROWSER", "WEBVIEW"]
    """URL open application, browser or webview."""

    url: str

    webview_view_mode: Literal["WEBVIEW_VIEW_MODE_UNSPECIFIED", "FULL", "HALF", "TALL"]

    description: Optional[str] = None
    """Accessbility description for webview."""


class ActionViewLocationActionLatLong(BaseModel):
    latitude: float
    """The latitude in degrees. It must be in the range [-90.0, +90.0]."""

    longitude: float
    """The longitude in degrees. It must be in the range [-180.0, +180.0]."""


class ActionViewLocationAction(BaseModel):
    """Opens the user's default map app and selects the agent-specified location."""

    label: Optional[str] = None
    """The label of the pin dropped"""

    lat_long: Optional[ActionViewLocationActionLatLong] = None

    query: Optional[str] = None
    """query string (Android only)"""


class Action(BaseModel):
    """When tapped, initiates the corresponding native action on the device."""

    create_calendar_event_action: Optional[ActionCreateCalendarEventAction] = None
    """
    Opens the user's default calendar app and starts the new calendar event flow
    with the agent-specified event data pre-filled.
    """

    dial_action: Optional[ActionDialAction] = None
    """
    Opens the user's default dialer app with the agent-specified phone number filled
    in.
    """

    fallback_url: Optional[str] = None
    """Fallback URL to use if a client doesn't support a suggested action.

    Fallback URLs open in new browser windows. Maximum 2048 characters.
    """

    open_url_action: Optional[ActionOpenURLAction] = None
    """Opens the user's default web browser app to the specified URL."""

    postback_data: Optional[str] = None
    """
    Payload (base64 encoded) that will be sent to the agent in the user event that
    results when the user taps the suggested action. Maximum 2048 characters.
    """

    share_location_action: Optional[Dict[str, object]] = None
    """
    Opens the RCS app's location chooser so the user can pick a location to send
    back to the agent.
    """

    text: Optional[str] = None
    """Text that is shown in the suggested action. Maximum 25 characters."""

    view_location_action: Optional[ActionViewLocationAction] = None
    """Opens the user's default map app and selects the agent-specified location."""


class Reply(BaseModel):
    postback_data: Optional[str] = None
    """
    Payload (base64 encoded) that will be sent to the agent in the user event that
    results when the user taps the suggested action. Maximum 2048 characters.
    """

    text: Optional[str] = None
    """Text that is shown in the suggested reply (maximum 25 characters)"""


class RcsSuggestion(BaseModel):
    action: Optional[Action] = None
    """When tapped, initiates the corresponding native action on the device."""

    reply: Optional[Reply] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ActionAddAIAssistantMessagesParams",
    "Message",
    "MessageUserMessage",
    "MessageAssistantMessage",
    "MessageAssistantMessageToolCall",
    "MessageAssistantMessageToolCallFunction",
    "MessageToolMessage",
    "MessageSystemMessage",
    "MessageDeveloperMessage",
]


class ActionAddAIAssistantMessagesParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    messages: Iterable[Message]
    """The messages to add to the conversation."""


class MessageUserMessage(TypedDict, total=False):
    """Messages sent by an end user"""

    content: Required[str]
    """The contents of the user message."""

    role: Required[Literal["user"]]
    """The role of the messages author, in this case `user`."""

    metadata: object
    """Metadata to add to the message"""


class MessageAssistantMessageToolCallFunction(TypedDict, total=False):
    """The function that the model called."""

    name: Required[str]
    """The name of the function to call."""


class MessageAssistantMessageToolCall(TypedDict, total=False):
    """A call to a function tool created by the model."""

    id: Required[str]
    """The ID of the tool call."""

    function: Required[MessageAssistantMessageToolCallFunction]
    """The function that the model called."""

    type: Required[Literal["function"]]
    """The type of the tool. Currently, only `function` is supported."""


class MessageAssistantMessage(TypedDict, total=False):
    """Messages sent by the model in response to user messages."""

    role: Required[Literal["assistant"]]
    """The role of the messages author, in this case `assistant`."""

    content: str
    """The contents of the assistant message. Required unless `tool_calls`"""

    metadata: object
    """Metadata to add to the message"""

    tool_calls: Iterable[MessageAssistantMessageToolCall]
    """The tool calls generated by the model, such as function calls."""


class MessageToolMessage(TypedDict, total=False):
    content: Required[str]
    """The contents of the tool message."""

    role: Required[Literal["tool"]]
    """The role of the messages author, in this case `tool`."""

    tool_call_id: Required[str]
    """Tool call that this message is responding to."""

    metadata: object
    """Metadata to add to the message"""


class MessageSystemMessage(TypedDict, total=False):
    """
    Developer-provided instructions that the model should follow, regardless of messages sent by the user.
    """

    content: Required[str]
    """The contents of the system message."""

    role: Required[Literal["system"]]
    """The role of the messages author, in this case `system`."""

    metadata: object
    """Metadata to add to the message"""


class MessageDeveloperMessage(TypedDict, total=False):
    """
    Developer-provided instructions that the model should follow, regardless of messages sent by the user.
    """

    content: Required[str]
    """The contents of the developer message."""

    role: Required[Literal["developer"]]
    """The role of the messages author, in this case developer."""

    metadata: object
    """Metadata to add to the message"""


Message: TypeAlias = Union[
    MessageUserMessage, MessageAssistantMessage, MessageToolMessage, MessageSystemMessage, MessageDeveloperMessage
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AssistantImportsParams"]


class AssistantImportsParams(TypedDict, total=False):
    api_key_ref: Required[str]
    """Integration secret pointer that refers to the API key for the external provider.

    This should be an identifier for an integration secret created via
    /v2/integration_secrets.
    """

    provider: Required[Literal["elevenlabs", "vapi", "retell"]]
    """The external provider to import assistants from."""

    import_ids: SequenceNotStr[str]
    """Optional list of assistant IDs to import from the external provider.

    If not provided, all assistants will be imported.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TelephonyCredentialListParams", "Filter"]


class TelephonyCredentialListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[tag], filter[name], filter[status], filter[resource_id],
    filter[sip_username]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[tag], filter[name], filter[status], filter[resource_id], filter[sip_username]
    """

    name: str
    """Filter by name"""

    resource_id: str
    """Filter by resource_id"""

    sip_username: str
    """Filter by sip_username"""

    status: str
    """Filter by status"""

    tag: str
    """Filter by tag"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from ..assistants.tests.test_suites.meta import Meta

__all__ = ["MessageListResponse", "Data", "DataToolCall", "DataToolCallFunction"]


class DataToolCallFunction(BaseModel):
    arguments: str
    """JSON-formatted arguments to pass to the function."""

    name: str
    """Name of the function to call."""


class DataToolCall(BaseModel):
    id: str
    """Unique identifier for the tool call."""

    function: DataToolCallFunction

    type: Literal["function"]
    """Type of the tool call."""


class Data(BaseModel):
    role: Literal["user", "assistant", "tool"]
    """The role of the message sender."""

    text: str
    """The message content. Can be null for tool calls."""

    created_at: Optional[datetime] = None
    """The datetime the message was created on the conversation.

    This does not necesarily correspond to the time the message was sent. The best
    field to use to determine the time the end user experienced the message is
    `sent_at`.
    """

    sent_at: Optional[datetime] = None
    """The datetime the message was sent to the end user."""

    tool_calls: Optional[List[DataToolCall]] = None
    """Optional tool calls made by the assistant."""


class MessageListResponse(BaseModel):
    data: List[Data]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    DtmfType,
    EncryptedMedia,
    TransportProtocol,
    WebhookAPIVersion,
    AnchorsiteOverride,
    fqdn_connection_list_params,
    fqdn_connection_create_params,
    fqdn_connection_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.dtmf_type import DtmfType
from ..types.encrypted_media import EncryptedMedia
from ..types.fqdn_connection import FqdnConnection
from ..types.inbound_fqdn_param import InboundFqdnParam
from ..types.transport_protocol import TransportProtocol
from ..types.anchorsite_override import AnchorsiteOverride
from ..types.outbound_fqdn_param import OutboundFqdnParam
from ..types.webhook_api_version import WebhookAPIVersion
from ..types.connection_rtcp_settings_param import ConnectionRtcpSettingsParam
from ..types.fqdn_connection_create_response import FqdnConnectionCreateResponse
from ..types.fqdn_connection_delete_response import FqdnConnectionDeleteResponse
from ..types.fqdn_connection_update_response import FqdnConnectionUpdateResponse
from ..types.fqdn_connection_retrieve_response import FqdnConnectionRetrieveResponse
from ..types.shared_params.connection_jitter_buffer import ConnectionJitterBuffer
from ..types.shared_params.connection_noise_suppression_details import ConnectionNoiseSuppressionDetails

__all__ = ["FqdnConnectionsResource", "AsyncFqdnConnectionsResource"]


class FqdnConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FqdnConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return FqdnConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FqdnConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return FqdnConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        connection_name: str,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundFqdnParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        microsoft_teams_sbc: bool | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundFqdnParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: TransportProtocol | Omit = omit,
        webhook_api_version: WebhookAPIVersion | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionCreateResponse:
        """
        Creates a FQDN connection.

        Args:
          connection_name: A user-assigned name to help manage the connection.

          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          microsoft_teams_sbc: When enabled, the connection will be created for Microsoft Teams Direct Routing.
              A \\**.mstsbc.telnyx.tech FQDN will be created for the connection automatically.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/fqdn_connections",
            body=maybe_transform(
                {
                    "connection_name": connection_name,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "microsoft_teams_sbc": microsoft_teams_sbc,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                fqdn_connection_create_params.FqdnConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionRetrieveResponse:
        """
        Retrieves the details of an existing FQDN connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fqdn_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundFqdnParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundFqdnParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: TransportProtocol | Omit = omit,
        webhook_api_version: WebhookAPIVersion | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionUpdateResponse:
        """
        Updates settings of an existing FQDN connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          connection_name: A user-assigned name to help manage the connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer that the sender and receiver negotiate T38
              directly when both are on the Telnyx network. If this is disabled, Telnyx will
              be able to use T38 on just one leg of the call according to each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/fqdn_connections/{id}",
            body=maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                fqdn_connection_update_params.FqdnConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: fqdn_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[FqdnConnection]:
        """
        Returns a list of your FQDN connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/fqdn_connections",
            page=SyncDefaultFlatPagination[FqdnConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    fqdn_connection_list_params.FqdnConnectionListParams,
                ),
            ),
            model=FqdnConnection,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionDeleteResponse:
        """
        Deletes an FQDN connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/fqdn_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionDeleteResponse,
        )


class AsyncFqdnConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFqdnConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFqdnConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFqdnConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncFqdnConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        connection_name: str,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundFqdnParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        microsoft_teams_sbc: bool | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundFqdnParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: TransportProtocol | Omit = omit,
        webhook_api_version: WebhookAPIVersion | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionCreateResponse:
        """
        Creates a FQDN connection.

        Args:
          connection_name: A user-assigned name to help manage the connection.

          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          microsoft_teams_sbc: When enabled, the connection will be created for Microsoft Teams Direct Routing.
              A \\**.mstsbc.telnyx.tech FQDN will be created for the connection automatically.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/fqdn_connections",
            body=await async_maybe_transform(
                {
                    "connection_name": connection_name,
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "microsoft_teams_sbc": microsoft_teams_sbc,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                fqdn_connection_create_params.FqdnConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionRetrieveResponse:
        """
        Retrieves the details of an existing FQDN connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fqdn_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundFqdnParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        jitter_buffer: ConnectionJitterBuffer | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ConnectionNoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundFqdnParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: TransportProtocol | Omit = omit,
        webhook_api_version: WebhookAPIVersion | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionUpdateResponse:
        """
        Updates settings of an existing FQDN connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          connection_name: A user-assigned name to help manage the connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          jitter_buffer: Configuration options for Jitter Buffer. Enables Jitter Buffer for RTP streams
              of SIP Trunking calls. The feature is off unless enabled. You may define min and
              max values in msec for customized buffering behaviors. Larger values add latency
              but tolerate more jitter, while smaller values reduce latency but are more
              sensitive to jitter and reordering.

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer that the sender and receiver negotiate T38
              directly when both are on the Telnyx network. If this is disabled, Telnyx will
              be able to use T38 on just one leg of the call according to each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/fqdn_connections/{id}",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "jitter_buffer": jitter_buffer,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                fqdn_connection_update_params.FqdnConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: fqdn_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FqdnConnection, AsyncDefaultFlatPagination[FqdnConnection]]:
        """
        Returns a list of your FQDN connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/fqdn_connections",
            page=AsyncDefaultFlatPagination[FqdnConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    fqdn_connection_list_params.FqdnConnectionListParams,
                ),
            ),
            model=FqdnConnection,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FqdnConnectionDeleteResponse:
        """
        Deletes an FQDN connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/fqdn_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FqdnConnectionDeleteResponse,
        )


class FqdnConnectionsResourceWithRawResponse:
    def __init__(self, fqdn_connections: FqdnConnectionsResource) -> None:
        self._fqdn_connections = fqdn_connections

        self.create = to_raw_response_wrapper(
            fqdn_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            fqdn_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            fqdn_connections.update,
        )
        self.list = to_raw_response_wrapper(
            fqdn_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            fqdn_connections.delete,
        )


class AsyncFqdnConnectionsResourceWithRawResponse:
    def __init__(self, fqdn_connections: AsyncFqdnConnectionsResource) -> None:
        self._fqdn_connections = fqdn_connections

        self.create = async_to_raw_response_wrapper(
            fqdn_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            fqdn_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            fqdn_connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            fqdn_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            fqdn_connections.delete,
        )


class FqdnConnectionsResourceWithStreamingResponse:
    def __init__(self, fqdn_connections: FqdnConnectionsResource) -> None:
        self._fqdn_connections = fqdn_connections

        self.create = to_streamed_response_wrapper(
            fqdn_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            fqdn_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            fqdn_connections.update,
        )
        self.list = to_streamed_response_wrapper(
            fqdn_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            fqdn_connections.delete,
        )


class AsyncFqdnConnectionsResourceWithStreamingResponse:
    def __init__(self, fqdn_connections: AsyncFqdnConnectionsResource) -> None:
        self._fqdn_connections = fqdn_connections

        self.create = async_to_streamed_response_wrapper(
            fqdn_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            fqdn_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            fqdn_connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            fqdn_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            fqdn_connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import TypedDict

__all__ = ["VerificationCodeVerifyParams", "VerificationCode"]


class VerificationCodeVerifyParams(TypedDict, total=False):
    verification_codes: Iterable[VerificationCode]


class VerificationCode(TypedDict, total=False):
    code: str

    phone_number: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PortingOrderRetrieveSubRequestResponse", "Data"]


class Data(BaseModel):
    port_request_id: Optional[str] = None
    """Identifies the Port Request associated with the Porting Order"""

    sub_request_id: Optional[str] = None
    """Identifies the Sub Request associated with the Porting Order"""


class PortingOrderRetrieveSubRequestResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_machine_detection_ended import CallMachineDetectionEnded

__all__ = ["CallMachineDetectionEndedWebhookEvent"]


class CallMachineDetectionEndedWebhookEvent(BaseModel):
    data: Optional[CallMachineDetectionEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .auto_resp_config import AutoRespConfig as AutoRespConfig
from .auto_resp_config_response import AutoRespConfigResponse as AutoRespConfigResponse
from .autoresp_config_list_params import AutorespConfigListParams as AutorespConfigListParams
from .autoresp_config_create_params import AutorespConfigCreateParams as AutorespConfigCreateParams
from .autoresp_config_list_response import AutorespConfigListResponse as AutorespConfigListResponse
from .autoresp_config_update_params import AutorespConfigUpdateParams as AutorespConfigUpdateParams
from .autoresp_config_delete_response import AutorespConfigDeleteResponse as AutorespConfigDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .conversation import Conversation

__all__ = ["ConversationUpdateResponse"]


class ConversationUpdateResponse(BaseModel):
    data: Optional[Conversation] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IPCreateParams"]


class IPCreateParams(TypedDict, total=False):
    ip_address: Required[str]
    """IP adddress represented by this resource."""

    connection_id: str
    """ID of the IP Connection to which this IP should be attached."""

    port: int
    """Port to use when connecting to this IP."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .cnam_listing_param import CnamListingParam
from .call_recording_param import CallRecordingParam
from .media_features_param import MediaFeaturesParam
from .call_forwarding_param import CallForwardingParam

__all__ = ["VoiceUpdateParams"]


class VoiceUpdateParams(TypedDict, total=False):
    call_forwarding: CallForwardingParam
    """The call forwarding settings for a phone number."""

    call_recording: CallRecordingParam
    """The call recording settings for a phone number."""

    caller_id_name_enabled: bool
    """Controls whether the caller ID name is enabled for this phone number."""

    cnam_listing: CnamListingParam
    """The CNAM listing settings for a phone number."""

    inbound_call_screening: Literal["disabled", "reject_calls", "flag_calls"]
    """
    The inbound_call_screening setting is a phone number configuration option
    variable that allows users to configure their settings to block or flag
    fraudulent calls. It can be set to disabled, reject_calls, or flag_calls. This
    feature has an additional per-number monthly cost associated with it.
    """

    media_features: MediaFeaturesParam
    """The media features settings for a phone number."""

    tech_prefix_enabled: bool
    """Controls whether a tech prefix is enabled for this phone number."""

    translated_number: str
    """
    This field allows you to rewrite the destination number of an inbound call
    before the call is routed to you. The value of this field may be any
    alphanumeric value, and the value will replace the number originally dialed.
    """

    usage_payment_method: Literal["pay-per-minute", "channel"]
    """
    Controls whether a number is billed per minute or uses your concurrent channels.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["GlobalIPProtocolListResponse", "Data"]


class Data(BaseModel):
    code: Optional[str] = None
    """The Global IP Protocol code."""

    name: Optional[str] = None
    """A name for Global IP Protocol."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class GlobalIPProtocolListResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .insight_template import InsightTemplate

__all__ = ["InsightTemplateDetail"]


class InsightTemplateDetail(BaseModel):
    data: InsightTemplate



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CampaignGetMnoMetadataResponse", "MnoMetadataItem10999"]


class MnoMetadataItem10999(BaseModel):
    min_msg_samples: int = FieldInfo(alias="minMsgSamples")

    mno: str

    mno_review: bool = FieldInfo(alias="mnoReview")

    mno_support: bool = FieldInfo(alias="mnoSupport")

    no_embedded_link: bool = FieldInfo(alias="noEmbeddedLink")

    no_embedded_phone: bool = FieldInfo(alias="noEmbeddedPhone")

    qualify: bool

    req_subscriber_help: bool = FieldInfo(alias="reqSubscriberHelp")

    req_subscriber_optin: bool = FieldInfo(alias="reqSubscriberOptin")

    req_subscriber_optout: bool = FieldInfo(alias="reqSubscriberOptout")


class CampaignGetMnoMetadataResponse(BaseModel):
    mno_metadata_item_10999: Optional[MnoMetadataItem10999] = FieldInfo(alias="10999", default=None)

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.doc_reqs_requirement_type import DocReqsRequirementType

__all__ = ["RequirementTypeRetrieveResponse"]


class RequirementTypeRetrieveResponse(BaseModel):
    data: Optional[DocReqsRequirementType] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncPerPagePagination, AsyncPerPagePagination
from telnyx.types.legacy.reporting.usage_reports import (
    MessagingCreateResponse,
    MessagingDeleteResponse,
    MessagingRetrieveResponse,
    MdrUsageReportResponseLegacy,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessaging:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.create(
            aggregation_type=0,
        )
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.create(
            aggregation_type=0,
            end_time=parse_datetime("2020-01-02T00:00:00Z"),
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            profiles=["3fa85f64-5717-4562-b3fc-2c963f66afa6", "7d4e3f8a-9b2c-4e1d-8f5a-1a2b3c4d5e6f"],
            select_all_managed_accounts=True,
            start_time=parse_datetime("2020-01-01T00:00:00Z"),
        )
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.messaging.with_raw_response.create(
            aggregation_type=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = response.parse()
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.messaging.with_streaming_response.create(
            aggregation_type=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = response.parse()
            assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.messaging.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = response.parse()
        assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.messaging.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = response.parse()
            assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.usage_reports.messaging.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.list()
        assert_matches_type(SyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.list(
            page=1,
            per_page=1,
        )
        assert_matches_type(SyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.messaging.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = response.parse()
        assert_matches_type(SyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.messaging.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = response.parse()
            assert_matches_type(SyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        messaging = client.legacy.reporting.usage_reports.messaging.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.legacy.reporting.usage_reports.messaging.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = response.parse()
        assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.legacy.reporting.usage_reports.messaging.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = response.parse()
            assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.usage_reports.messaging.with_raw_response.delete(
                "",
            )


class TestAsyncMessaging:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.create(
            aggregation_type=0,
        )
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.create(
            aggregation_type=0,
            end_time=parse_datetime("2020-01-02T00:00:00Z"),
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            profiles=["3fa85f64-5717-4562-b3fc-2c963f66afa6", "7d4e3f8a-9b2c-4e1d-8f5a-1a2b3c4d5e6f"],
            select_all_managed_accounts=True,
            start_time=parse_datetime("2020-01-01T00:00:00Z"),
        )
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.create(
            aggregation_type=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = await response.parse()
        assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.messaging.with_streaming_response.create(
            aggregation_type=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = await response.parse()
            assert_matches_type(MessagingCreateResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = await response.parse()
        assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.messaging.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = await response.parse()
            assert_matches_type(MessagingRetrieveResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.list()
        assert_matches_type(AsyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.list(
            page=1,
            per_page=1,
        )
        assert_matches_type(AsyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = await response.parse()
        assert_matches_type(AsyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.messaging.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = await response.parse()
            assert_matches_type(AsyncPerPagePagination[MdrUsageReportResponseLegacy], messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        messaging = await async_client.legacy.reporting.usage_reports.messaging.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging = await response.parse()
        assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.usage_reports.messaging.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging = await response.parse()
            assert_matches_type(MessagingDeleteResponse, messaging, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.usage_reports.messaging.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["WellKnownRetrieveAuthorizationServerMetadataResponse"]


class WellKnownRetrieveAuthorizationServerMetadataResponse(BaseModel):
    authorization_endpoint: Optional[str] = None
    """Authorization endpoint URL"""

    code_challenge_methods_supported: Optional[List[str]] = None
    """Supported PKCE code challenge methods"""

    grant_types_supported: Optional[List[str]] = None
    """Supported grant types"""

    introspection_endpoint: Optional[str] = None
    """Token introspection endpoint URL"""

    issuer: Optional[str] = None
    """Authorization server issuer URL"""

    jwks_uri: Optional[str] = None
    """JWK Set endpoint URL"""

    registration_endpoint: Optional[str] = None
    """Dynamic client registration endpoint URL"""

    response_types_supported: Optional[List[str]] = None
    """Supported response types"""

    scopes_supported: Optional[List[str]] = None
    """Supported OAuth scopes"""

    token_endpoint: Optional[str] = None
    """Token endpoint URL"""

    token_endpoint_auth_methods_supported: Optional[List[str]] = None
    """Supported token endpoint authentication methods"""
